﻿using System;
using System.Diagnostics;
using DerpingDrivers.Exceptions;
using ExceptionReporting;
using Microsoft.Win32;

namespace DerpingDrivers.Util
{
    public static class BcdHelper
    {
        /// <summary>
        ///     Gets or sets the current value if BCDE_LIBRARY_TYPE_ALLOW_PRERELEASE_SIGNATURES from the default boot entry.
        /// </summary>
        public static bool AllowPrereleaseSignatures
        {
            get
            {
                var bootmgrDefaultGuid = (string) Registry.GetValue(
                    @"HKEY_LOCAL_MACHINE\BCD00000000\Objects\{9DEA862C-5CDD-4E70-ACC1-F32B344D4795}\Elements\23000003",
                    "Element", null);

                var allowPrereleaseSignatures = (byte[]) Registry.GetValue(
                    $@"HKEY_LOCAL_MACHINE\BCD00000000\Objects\{bootmgrDefaultGuid}\Elements\16000049",
                    "Element", default(byte[]));

                return allowPrereleaseSignatures != null && int.Parse(allowPrereleaseSignatures[0].ToString()) > 0;
            }

            set
            {
                try
                {
                    if (value)
                    {
                        // NOTE: haven't found an API for this (yet), so system() we go!
                        var cmd = new Process
                        {
                            StartInfo =
                            {
                                FileName = @"C:\WINDOWS\system32\bcdedit.exe",
                                Arguments = "-set TESTSIGNING ON",
                                WindowStyle = ProcessWindowStyle.Hidden,
                                CreateNoWindow = true,
                                UseShellExecute = false
                            }
                        };

                        cmd.Start();
                        cmd.WaitForExit();

                        if (cmd.ExitCode != 0)
                            throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't enable TESTSIGNING");
                    }
                    else
                    {
                        // NOTE: haven't found an API for this (yet), so system() we go!
                        var cmd = new Process
                        {
                            StartInfo =
                            {
                                FileName = @"C:\WINDOWS\system32\bcdedit.exe",
                                Arguments = "-set TESTSIGNING OFF",
                                WindowStyle = ProcessWindowStyle.Hidden,
                                CreateNoWindow = true,
                                UseShellExecute = false
                            }
                        };

                        cmd.Start();
                        cmd.WaitForExit();

                        if (cmd.ExitCode != 0)
                            throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't disable TESTSIGNING");
                    }
                }
                catch (Exception ex)
                {
                    var er = new ExceptionReporter
                    {
                        Config =
                        {
                            AppName = "DerpingDrivers",
                            SendMethod = ReportSendMethod.None
                        }
                    };
                    er.Show(ex);
                }
            }
        }
    }
}